// In this sketch we use some analog potentiometers to set the parameters of the acceleration curve
// This is a test sketch for Arduino UNO and Duemilanove to test and setup the digital potentiometer
// Arduino Shield.
// Created on March 2015 by balearicdynamics@gmail.com http://www.balearicdynamics.com

// Include SPI Library
#include <SPI.h>
// Include the Streaming library for better serial output
#include <Streaming.h>

#define MAXPOT 6  // Max number of potentiometers
#define SERIALMON  // Undef SERIALMON to avoid serial output. For testing purposes only

// Steps divider from analog converter
#define ANALOGDIVIDER 1024.00

// Number of samples on the same analog potentiometer to
// calculate the average value.
#define ANALOGSAMPLES  5

// Steps divider of the digital potentiometer
#define DIGITALDIVIDER 255

// Set the analog pins array conected to the potentiometers
const int AnalogPotPort[] = { 0, 1, 2, 3, 4, 5 };

// Current motors setting.
// These settings can be changed while the sketch is running,
// at startup the last set value from the potentiometers is read.
int configSettings[MAXPOT];

// Constants defining the usage of the analog potentiometers
#define SET_MOTOR_MIN    0
#define SET_MOTOR_MAX    1
#define SET_CURVE_SMOOTHING  2
#define SET_MOTOR_PAUSE  3

// Digital potentiometer used to control the motor speed
#define DIGITAL_MOTOR_CONTROL  0

// set the slave select for the digital pot
const int slaveSelectPin = 10;

// Initial potentiometer value
int pValue = 0;

// Your setup initialization here
void setup() 
{
  
        // set the output pins
        pinMode (slaveSelectPin, OUTPUT);

        // initialize SPI:
        SPI.begin();
        
        // Preset all the digital potentiometer values to max
        // (Higher impedance, minimum voltage)
        for(int k = 0; k < MAXPOT; k++) {
          digitalPotWrite(k, 0);
        }
        
#ifdef SERIALMON        
        // Initialize the serial (for testing purposes)
        Serial.begin(9600);
        Serial << "START" << endl;
#endif

        // Load the startup analog potentiometers settings
        loadSettings();

} // end

// Main application loop
// This sample focus the attention on the digital potentiometer management.
// Several improvements can be added, e.g. the use of one interrupt instead
// a delay() call to cycle the SPI frequency.
void loop(void) {
    
  // Start a motor acceleration cycle
  for(int i = mapValue(configSettings[SET_MOTOR_MIN]); i < mapValue(configSettings[SET_MOTOR_MAX]); i++) {
    // Write the value to the digital potentiometer motor control
    digitalPotWrite(DIGITAL_MOTOR_CONTROL, i);
    // Cycle pause
    delay(configSettings[SET_CURVE_SMOOTHING]);
  } // End
  
  // Start a motor deceleration cycle
  for(int i = mapValue(configSettings[SET_MOTOR_MAX]); i > mapValue(configSettings[SET_MOTOR_MIN]); i--) {
    // Write the value to the digital potentiometer motor control
    digitalPotWrite(DIGITAL_MOTOR_CONTROL, i);
    // Cycle pause
    delay(configSettings[SET_CURVE_SMOOTHING]);
  } // End
  
  // Wait for the next cycle
  delay(configSettings[SET_MOTOR_PAUSE]);
  
  loadSettings();
}

// =================================================================================

  // Convert the value acquire on the scale of the analog port
  // to the corresponding digital scale.
  // In our case as we read a variation of max 1024 steps in the 
  // analog port we should reduce the resolution to 256th of step.
  int mapValue(int aVal) {
   return map(aVal, 0, ANALOGDIVIDER, 0, DIGITALDIVIDER);
  } // end

// Read the analog potentiometers values and update the potentiometers array
// For every potentiometer the same value is read ANALOGSAMPLES times and
// it is stored the average value to normalize the reading and avoid
// in a very simple way values fluctuations.
void loadSettings() {
  int pValue;
  
  // Analog potentometers read cycle
  for(int i = 0; i < MAXPOT; i++) {
    pValue = 0; // initializes the potentiometer reading
    
    // Analog sampling cycle for every potentiometer
    for(int j = 0; j < ANALOGSAMPLES; j++) {
      pValue += analogRead(AnalogPotPort[i]);
    } // sampling cycle
    
    // Calculate the average value and save it in the array
    configSettings[i] = pValue / ANALOGSAMPLES;
  } // Analog reading cycle
  
  // Check for the limits. If the Max value is smaller than min, the
  // two settings are inverted.
  if(configSettings[SET_MOTOR_MIN] > configSettings[SET_MOTOR_MAX]) {
    int temp = configSettings[SET_MOTOR_MIN];
    configSettings[SET_MOTOR_MIN] = configSettings[SET_MOTOR_MAX];
    configSettings[SET_MOTOR_MAX] = temp;
  }
  
  // Avoid 0 a minim limit. The starting value can't be less than 1
  if(configSettings[SET_MOTOR_MIN] == 0) {
    configSettings[SET_MOTOR_MIN] = 1;
  }
  
#ifdef SERIALMON        
  Serial << 
        "---------------------------------------------------" << endl <<
        "Settings: " << endl << 
        "---------------------------------------------------" << endl <<
        SET_MOTOR_MIN << ".SET_MOTOR_MIN = " << mapValue(configSettings[SET_MOTOR_MIN]) << endl <<
        SET_MOTOR_MAX << ".SET_MOTOR_MAX = " << mapValue(configSettings[SET_MOTOR_MAX]) << endl <<
        SET_CURVE_SMOOTHING << ".SET_CURVE_SMOOTHING = " << configSettings[SET_CURVE_SMOOTHING] << endl <<
        SET_MOTOR_PAUSE << ".SET_MOTOR_PAUSE = " << configSettings[SET_MOTOR_PAUSE] << endl << endl;
#endif
} // end

// Write the resistance value to the selected digital potentiometer
// The resistance value is converted to the range accepted by the
// digital potentiometer.
void digitalPotWrite(int address, int value) {
  // take the SS pin low to select the chip:
  digitalWrite(slaveSelectPin,LOW);
  //  send in the address and value via SPI:
  SPI.transfer(address);
  SPI.transfer(value);
  // take the SS pin high to de-select the chip:
  digitalWrite(slaveSelectPin,HIGH); 
} // end
